/*
Copyright (C) 2001-2002 Tomas Jakobsson.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include	<dsound.h>
#include	"Sound.h"

#define		WIN32_LEAN_AND_MEAN

HWND				MRS_Window				= NULL;
LPDIRECTSOUND		MRS_DirectSound			= NULL;
LPDIRECTSOUNDBUFFER	MRS_BufferPrimary		= NULL;
LPDIRECTSOUNDBUFFER	SampleBuffer[32];
int					SampleVolume[32];

BOOL MRS_Init (HWND Window)
{
	DSBUFFERDESC	BufferDesc;   
	
	MRS_Window = Window;

	if (FAILED (DirectSoundCreate (NULL, &MRS_DirectSound, NULL)))
		return FALSE;

	// Set DirectSound coop level 
	if (FAILED (MRS_DirectSound->lpVtbl->SetCooperativeLevel (MRS_DirectSound, Window, DSSCL_PRIORITY)))
		return FALSE;

	BufferDesc.dwSize			= sizeof(DSBUFFERDESC);
	BufferDesc.dwFlags			= DSBCAPS_PRIMARYBUFFER | DSBCAPS_LOCHARDWARE;
	BufferDesc.dwBufferBytes	= 0;
	BufferDesc.dwReserved		= 0 ;
	BufferDesc.lpwfxFormat		= NULL;

	return TRUE;
}

void MRS_Shutdown (void)
{
	if (MRS_DirectSound)
		MRS_DirectSound->lpVtbl->Release (MRS_DirectSound);

	MRS_DirectSound = NULL;
}

BOOL MRS_SampleLoad (int number, char *filename)
{
	DSBUFFERDESC		DSBD;
	HMMIO				WAVE;
	WAVEFORMATEX		WAVEF;
	MMCKINFO			Child;
	MMCKINFO			Parent;
	byte				*Data;
	byte				*Write1;
	byte				*Write2;
	DWORD				WriteLength1;
	DWORD				WriteLength2;

	if ((WAVE = mmioOpen (filename, NULL, MMIO_READ | MMIO_ALLOCBUF )) == NULL)
	{
		return FALSE;
	}

	Parent.fccType = mmioFOURCC ('W', 'A', 'V', 'E');
	if (mmioDescend (WAVE, &Parent, NULL, MMIO_FINDRIFF))
	{
		mmioClose (WAVE, 0);
		return FALSE;
	}
	
	Child.ckid = mmioFOURCC ('f', 'm', 't', ' ');
	if (mmioDescend (WAVE, &Child, &Parent, 0))
	{
		mmioClose (WAVE, 0);
		return FALSE;
	}

	if (mmioRead (WAVE, (char *)&WAVEF, sizeof (WAVEFORMATEX)) != sizeof (WAVEFORMATEX))
	{
		mmioClose (WAVE, 0);
		return FALSE;
	}

	if (WAVEF.wFormatTag != WAVE_FORMAT_PCM)
	{
		mmioClose (WAVE, 0);
		return FALSE;
	}

	if (mmioAscend (WAVE, &Child, 0))
	{
		mmioClose (WAVE, 0);
		return FALSE;
	}

	Child.ckid = mmioFOURCC ('d', 'a', 't', 'a');
	if (mmioDescend (WAVE, &Child, &Parent, MMIO_FINDCHUNK))
	{
		mmioClose (WAVE, 0);
		return FALSE;
	}

	Data = (byte *)malloc (Child.cksize);
	if (Data == NULL)
	{
		mmioClose (WAVE, 0);
		return FALSE;
	}

	// read the data, then close the file.
	mmioRead (WAVE, (char *)Data, Child.cksize);
	mmioClose(WAVE, 0);

	// create the sound buffer
	memset (&DSBD, 0, sizeof (DSBD));

	DSBD.dwSize			= sizeof (DSBUFFERDESC);
	DSBD.dwBufferBytes	= Child.cksize;
	DSBD.dwFlags		= DSBCAPS_STATIC | DSBCAPS_LOCSOFTWARE | DSBCAPS_GLOBALFOCUS | DSBCAPS_CTRLPAN | DSBCAPS_CTRLVOLUME | DSBCAPS_CTRLFREQUENCY;
	DSBD.lpwfxFormat	= &WAVEF;

	if (FAILED (MRS_DirectSound->lpVtbl->CreateSoundBuffer (MRS_DirectSound, &DSBD, &SampleBuffer[number], NULL)))
	{
		free (Data);
		return FALSE;
	}

	if (FAILED (SampleBuffer[number]->lpVtbl->Lock (SampleBuffer[number], 0, Child.cksize, (void **)&Write1, &WriteLength1, (void **)&Write2, &WriteLength2, DSBLOCK_FROMWRITECURSOR)))
	{
		free (Data);
		SampleBuffer[number]->lpVtbl->Release (SampleBuffer[number]);
		SampleBuffer[number] = NULL;
		return FALSE;
	}

	memcpy (Write1, Data, WriteLength1);
	memcpy (Write2, Data + WriteLength1, WriteLength2);
	free (Data);

	if (FAILED (SampleBuffer[number]->lpVtbl->Unlock (SampleBuffer[number], Write1, WriteLength1, Write2, WriteLength2)))
	{
		SampleBuffer[number]->lpVtbl->Release (SampleBuffer[number]);
		SampleBuffer[number] = NULL;
		return FALSE;
	}

	return TRUE;
}

void MRS_SampleFree (int number)
{
	if (SampleBuffer[number])
	{
		SampleBuffer[number]->lpVtbl->Stop (SampleBuffer[number]);
		SampleBuffer[number]->lpVtbl->Release (SampleBuffer[number]);
	}

	memset(&SampleBuffer[number], 0, sizeof (SampleBuffer[number]));
}

void MRS_SamplePlay (int number, int volume, BOOL loop)
{
	if (SampleBuffer[number])
	{
		if (SampleVolume[number] != volume)
		{
			SampleVolume[number] = volume;
			SampleBuffer[number]->lpVtbl->SetVolume (SampleBuffer[number], SampleVolume[number]);
		}

		SampleBuffer[number]->lpVtbl->Stop (SampleBuffer[number]);
		SampleBuffer[number]->lpVtbl->SetCurrentPosition (SampleBuffer[number], 0);

		if (loop)
		{
			SampleBuffer[number]->lpVtbl->Play (SampleBuffer[number], 0, 0, DSBPLAY_LOOPING);
		}
		else
		{
			SampleBuffer[number]->lpVtbl->Play (SampleBuffer[number], 0, 0, 0);
		}
	}
}

void MRS_SampleStop (int number)
{
	if (SampleBuffer[number])
	{
		SampleBuffer[number]->lpVtbl->Stop (SampleBuffer[number]);
	}
}

void MRS_SampleSetVolume (int number, int volume)
{
	if (SampleBuffer[number])
	{
		SampleBuffer[number]->lpVtbl->SetVolume (SampleBuffer[number], volume);
	}
}

void MRS_SampleSetPan (int number, int pan)
{
	if (SampleBuffer[number])
	{
		SampleBuffer[number]->lpVtbl->SetPan (SampleBuffer[number], pan);
	}
}

void MRS_SampleSetPitch (int number, int pitch)
{
	if (pitch < DSBFREQUENCY_MIN)
	{
		pitch = DSBFREQUENCY_MIN;
	}
	else if (pitch > DSBFREQUENCY_MAX)
	{
		pitch = DSBFREQUENCY_MAX;
	}

	if (SampleBuffer[number])
	{
		SampleBuffer[number]->lpVtbl->SetFrequency (SampleBuffer[number], pitch);
	}
}

int MRS_SampleGetPitch (int number)
{
	unsigned long	pitch;

	if (SampleBuffer[number])
	{
		SampleBuffer[number]->lpVtbl->GetFrequency (SampleBuffer[number], &pitch);
		return (int)pitch;
	}

	return 0;
}